@@ -9,7 +9,6 @@ use strict;
 package KronaTools;
 
 use Getopt::Long;
-Getopt::Long::Configure("no_ignore_case");
 use File::Basename;
 use File::Path;
 
@@ -36,7 +35,6 @@ our @EXPORT = qw
 	addByLineage
 	addByTaxID
 	addXML
-	classify
 	classifyBlast
 	default
 	getAccFromSeqID
@@ -92,8 +90,7 @@ my %options =
 	'standalone' => 1,
 	'taxCol' => 2,
 	'taxonomy' => $taxonomyDir,
-	'threshold' => 3,
-	'thresholdGeneric' => 0,
+	'threshold' => 3
 );
 
 # Option format codes to pass to GetOptions (and to be parsed for display).
@@ -114,14 +111,10 @@ my %optionFormats =
 		'e=i',
 	'factor' =>
 		'e=f',
-	'includeUnk' =>
-		'f',
 	'include' =>
 		'i',
-	'cellular' =>
-		'k',
 	'noRank' =>
-		'K',
+		'k',
 	'local' =>
 		'l',
 	'magCol' =>
@@ -154,8 +147,6 @@ my %optionFormats =
 		't=i',
 	'threshold' =>
 		't=f',
-	'thresholdGeneric' =>
-		't=f',
 	'taxonomy' =>
 		'tax=s',
 	'url' =>
@@ -182,7 +173,6 @@ my %optionTypes =
 my %optionDescriptions =
 (
 	'bitScore' => 'Use bit score for average scores instead of log[10] e-value.',
-	'cellular' => 'Show the "cellular organisms" taxon (collapsed by default).',
 	'combine' => 'Combine data from each file, rather than creating separate datasets within the chart.',
 	'depth' => 'Maximum depth of wedges to include in the chart.',
 	'ecCol' => 'Column of input files to use as EC number.',
@@ -191,13 +181,12 @@ my %optionDescriptions =
 	'hueGood' => 'Hue (0-360) for "good" scores.',
 	'percentIdentity' => 'Use percent identity for average scores instead of log[10] e-value.',
 	'include' => 'Include a wedge for queries with no hits.',
-	'includeUnk' => 'If any best hits have unknown accessions, force classification to root instead of ignoring them.',
 	'local' => 'Use resources from the local KronaTools installation instead of bundling them with charts (charts will be smaller but will only work on this computer).',
 	'magCol' => 'Column of input files to use as magnitude. If magnitude files are specified, their magnitudes will override those in this column.',
 	'minConfidence' => 'Minimum confidence. Each query sequence will only be added to taxa that were predicted with a confidence score of at least this value.',
 	'name' => 'Name of the highest level.',
 	'noMag' => 'Files do not have a field for quantity.',
-	'noRank' => 'Collapse assignments to taxa with ranks labeled "no rank" by moving up to parent.',
+	'noRank' => 'Allow assignments to taxa with ranks labeled "no rank" (instead of moving up to parent).',
 	'out' => 'Output file name.',
 	'phymm' => 'Input is phymm only (no confidence scores).',
 	'postUrl' => 'Url to send query IDs to (instead of listing them) for each wedge. The query IDs will be sent as a comma separated list in the POST variable "queries", with the current dataset index (from 0) in the POST variable "dataset". The url can include additional variables encoded via GET.',
@@ -209,7 +198,6 @@ my %optionDescriptions =
 	'taxCol' => 'Column of input files to use as taxonomy ID.',
 	'taxonomy' => 'Path to directory containing a taxonomy database to use.',
 	'threshold' => 'Threshold for bit score differences when determining "best" hits. Hits with scores that are within this distance of the highest score will be included when computing the lowest common ancestor (or picking randomly if -r is specified).',
-	'thresholdGeneric' => 'Threshold for score differences when determining "best" hits. Hits with scores that are within this distance of the highest score will be included when computing the lowest common ancestor (or picking randomly if -r is specified). If 0, only exact ties for the best hit are used.',
 	'url' => 'URL of Krona resources to use instead of bundling them with the chart (e.g. "http://krona.sourceforge.net"). Reduces size of charts and allows updates, though charts will not work without access to this URL.',
 	'verbose' => 'Verbose.'
 );
@@ -224,7 +212,6 @@ my %optionDescriptions =
 our %argumentNames =
 (
 	'blast' => 'blast_output',
-	'hits' => 'hits',
 	'magnitude' => 'magnitudes',
 	'metarep' => 'metarep_folder',
 	'name' => 'name',
@@ -239,12 +226,6 @@ our %argumentDescriptions =
 downloading from NCBI). If running BLAST locally, subject IDs in the local
 database must contain accession numbers, either bare or in the fourth field of
 the pipe-separated ("gi|12345|xx|ABC123.1|") format.',
-	'hits' =>
-'Tabular file whose fields are [query, subject, score]. Subject must be
-an accession or contain one in the fourth field of pipe notation (e.g.
-"gi|12345|xx|ABC123.1|". The subject and score can be omitted to
-include a query that has no hits, which will be assigned a taxonomy
-ID of -1.',
 	'magnitude' =>
 'Optional file listing query IDs with magnitudes, separated by tabs. This can
 be used to account for read length or contig depth to obtain a more accurate
@@ -264,14 +245,14 @@ default, the basename of the file will be used.',
 # Global constants #
 ####################
 
-our $version = '2.8.1';
+our $version = '2.7';
 my $javascriptVersion = '2.0';
 my $javascript = "src/krona-$javascriptVersion.js";
 my $hiddenImage = 'img/hidden.png';
 my $favicon = 'img/favicon.ico';
 my $loadingImage = 'img/loading.gif';
 my $logo = 'img/logo-med.png';
-my $taxonomyHrefBase = 'http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?mode=info&amp;id=';
+my $taxonomyHrefBase = 'http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?mode=info&id=';
 my $ecHrefBase = 'http://www.chem.qmul.ac.uk/iubmb/enzyme/EC';
 my $suppDirSuffix = '.files';
 my $suppEnableFile = 'enable.js';
@@ -279,15 +260,7 @@ my $fileTaxonomy = 'taxonomy.tab';
 my $fileTaxByAcc = 'all.accession2taxid.sorted';
 my $memberLimitDataset = 10000;
 my $memberLimitTotal = 100000;
-my $columns;
-if (defined($ENV{TERM}))
-{
-	$columns = `tput cols`;
-}
-else
-{
-	$columns = 80;
-}
+my $columns = `tput cols`;
 our $minEVal = -450;
 
 
@@ -595,11 +568,11 @@ sub addByTaxID
 	#
 	while
 	(
-		shouldCollapse($taxID) ||
-		$options{'depth'} && getTaxDepth($taxID) > $options{'depth'}
+		! $options{'noRank'} && $taxID > 1 && $taxRanks[$taxID] eq 'no rank' ||
+		$options{'depth'} && $taxDepths[$taxID] > $options{'depth'}
 	)
 	{
-		$taxID = getTaxParent($taxID);
+		$taxID = $taxParents[$taxID];
 	}
 	
 	# get parent recursively
@@ -611,7 +584,7 @@ sub addByTaxID
 	{
 		$parentID = getTaxParent($parentID);
 	}
-	while ( shouldCollapse($parentID) );
+	while (! $options{'noRank'} && $parentID > 1 && getTaxRank($parentID) eq 'no rank');
 	
 	#
 	if ( $parentID != 1 )#$taxID )
@@ -786,139 +759,12 @@ sub addXML
 	}
 }
 
-sub classify
-{
-	# taxonomically classifies generic hits based on LCA (or random selection)
-	# of 'best' hits.
-	#
-	# Options used: thresholdGeneric, include, percentIdentity, random, score
-	
-	my # parameters
-	(
-		$fileName, # file with tabular hits (query, subject, score)
-		
-		# hash refs to be populated with results (keyed by query ID)
-		#
-		$taxIDs,
-		$scores
-	) = @_;
-	
-	open HITS, "<$fileName" or ktDie("Could not open $fileName\n");
-	
-	my $lastQueryID;
-	my $topScore;
-	my $ties;
-	my $taxID;
-	my %lcaSet;
-	my $totalScore;
-	
-	while ( 1 )
-	{
-		my $line = <HITS>;
-		
-		chomp $line;
-		
-		my
-		(
-			$queryID,
-			$hitID,
-			$score
-		) = split /\t/, $line;
-		
-		if ( defined $queryID && ! defined $hitID )
-		{
-			$taxIDs->{$queryID} = -1;
-			$scores->{$queryID} = 0;
-			
-			next;
-		}
-		
-		if ( $queryID ne $lastQueryID )
-		{
-			if (  $ties )
-			{
-				# add the chosen hit from the last queryID
-				
-				if ( ! $options{'random'} )
-				{
-					$taxID = taxLowestCommonAncestor(keys %lcaSet)
-				}
-				
-				$taxIDs->{$lastQueryID} = $taxID;
-				$scores->{$lastQueryID} = $totalScore / $ties;
-			}
-			
-			$ties = 0;
-			$totalScore = 0;
-			%lcaSet = ();
-		}
-		
-		if ( ! defined $hitID )
-		{
-			last; # EOF
-		}
-		
-		my $acc = getAccFromSeqID($hitID);
-		
-		if ( ! defined $acc )
-		{
-			$lastQueryID = $queryID;
-			next;
-		}
-		
-		if # this is a 'best' hit if...
-		(
-			$queryID ne $lastQueryID || # new query ID (including null at EOF)
-			$score >= $topScore - $options{'thresholdGeneric'} # within score threshold
-		)
-		{
-			# add score for average
-			#
-			$totalScore += $score;
-			$ties++;
-			
-			if # use this hit if...
-			(
-				! $options{'random'} || # using LCA
-				$queryID ne $lastQueryID || # new query ID
-				int(rand($ties)) == 0 # randomly chosen to replace other hit
-			)
-			{
-				my $newTaxID = getTaxIDFromAcc($acc);
-				
-				if ( ! $newTaxID || ! taxIDExists($newTaxID) )
-				{
-					$newTaxID = 1;
-				}
-				
-				if ( $options{'random'} )
-				{
-					$taxID = $newTaxID;
-				}
-				else
-				{
-					$lcaSet{$newTaxID} = 1;
-				}
-			}
-		}
-		
-		if ( $queryID ne $lastQueryID )
-		{
-			$topScore = $score;
-		}
-		
-		$lastQueryID = $queryID;
-	}
-	
-	close HITS;
-}	
-
 sub classifyBlast
 {
 	# taxonomically classifies BLAST results based on LCA (or random selection)
 	# of 'best' hits.
 	#
-	# Options used: bitScore, factor, include, percentIdentity, random, score, includeUnk
+	# Options used: bitScore, factor, include, percentIdentity, random, score
 	
 	my # parameters
 	(
@@ -1011,32 +857,21 @@ sub classifyBlast
 			last; # EOF
 		}
 		
+		my $acc = getAccFromSeqID($hitID);
+		
+		if ( ! defined $acc )
+		{
+			$lastQueryID = $queryID;
+			next;
+		}
+		
 		if # this is a 'best' hit if...
 		(
-			$ties == 0 || # first hit
-			$bitScore >= $topScore - $options{'threshold'} || # within score threshold
+			$queryID ne $lastQueryID || # new query ID (including null at EOF)
+			$bitScore > $topScore - $options{'threshold'} || # within score threshold
 			$options{'factor'} && $eVal <= $options{'factor'} * $topEVal # within e-val factor
 		)
 		{
-			my $acc = getAccFromSeqID($hitID);
-			my $newTaxID = getTaxIDFromAcc($acc);
-			
-			if
-			(
-				! defined $acc ||
-				! $options{'includeUnk'} && (! $newTaxID || ! taxIDExists($newTaxID))
-			)
-			{
-				$lastQueryID = $queryID;
-				next;
-			}
-			
-			if ( $ties == 0 )
-			{
-				$topScore = $bitScore;
-				$topEVal = $eVal;
-			}
-			
 			# add score for average
 			#
 			if ( $options{'percentIdentity'} )
@@ -1069,7 +904,9 @@ sub classifyBlast
 				int(rand($ties)) == 0 # randomly chosen to replace other hit
 			)
 			{
-				if ( ! $newTaxID || ! taxIDExists($newTaxID) )
+				my $newTaxID = getTaxIDFromAcc($acc);
+				
+				if ( ! $newTaxID || ! defined $taxParents[$newTaxID] )
 				{
 					$newTaxID = 1;
 				}
@@ -1085,11 +922,15 @@ sub classifyBlast
 			}
 		}
 		
+		if ( $queryID ne $lastQueryID )
+		{
+			$topScore = $bitScore;
+			$topEVal = $eVal;
+		}
+		
 		$lastQueryID = $queryID;
 	}
 	
-	close BLAST;
-	
 	if ( $zeroEVal )
 	{
 		ktWarn("\"$fileName\" had e-values of 0. Approximated log[10] of 0 as $minEVal.");
@@ -1125,10 +966,10 @@ sub getAccFromSeqID
 		$acc = (split /\|/, $acc)[3];
 	}
 	
-	if ( $acc !~ /^\d+$/ && $acc !~ /^[A-Z\d]+_?[A-Z\d]+(\.\d+)?$/ )
+	if ( $acc !~ /^\d+$/ && $acc !~ /^[A-Z]+_?[A-Z]*\d+(\.\d+)?$/ )
 	{
 		$invalidAccs{$acc} = 1;
-		#return undef;
+		return undef;
 	}
 	
 	return $acc;
@@ -1224,11 +1065,6 @@ sub getTaxInfo
 	
 	$tax = int($tax);
 	
-	if ( $tax == 0 )
-	{
-		return ('', '', '', '', '');
-	}
-	
 	if ( defined $taxInfoByID{$tax} )
 	{
 		return @{$taxInfoByID{$tax}};
@@ -1457,7 +1293,7 @@ sub htmlHeader
 			$notFound = "Could not get resources from \\\"$options{'url'}\\\".";
 		}
 		
-		$script = indent(2) . "<script src=\"$path$javascript\" type=\"text/javascript\"></script>\n";
+		$script = indent(2) . "<script src=\"$path$javascript\"></script>\n";
 	}
 	
 	return
@@ -1467,13 +1303,13 @@ sub htmlHeader
 			indent(2) . "<meta charset=\"utf-8\"/>\n" .
 #			indent(2) . "<base href=\"$path\" target=\"_blank\"/>\n" .
 			indent(2) . "<link rel=\"shortcut icon\" href=\"$path$favicon\"/>\n" .
-			indent(2) . "<script id=\"notfound\" type=\"text/javascript\">window.onload=function(){document.body.innerHTML=\"$notFound\"}</script>\n" .
+			indent(2) . "<script id=\"notfound\">window.onload=function(){document.body.innerHTML=\"$notFound\"}</script>\n" .
 			$script .
 		indent(1) . "</head>\n" .
 		indent(1) . "<body>\n" .
-			indent(2) . "<img id=\"hiddenImage\" src=\"$path$hiddenImage\" style=\"display:none\" alt=\"Hidden Image\"/>\n" .
-			indent(2) . "<img id=\"loadingImage\" src=\"$path$loadingImage\" style=\"display:none\" alt=\"Loading Indicator\"/>\n" .
-			indent(2) . "<img id=\"logo\" src=\"$path$logo\" style=\"display:none\" alt=\"Logo of Krona\"/>\n" .
+			indent(2) . "<img id=\"hiddenImage\" src=\"$path$hiddenImage\" style=\"display:none\"/>\n" .
+			indent(2) . "<img id=\"loadingImage\" src=\"$path$loadingImage\" style=\"display:none\"/>\n" .
+			indent(2) . "<img id=\"logo\" src=\"$path$logo\" style=\"display:none\"/>\n" .
 			indent(2) . "<noscript>Javascript must be enabled to view this page.</noscript>\n" .
 			indent(2) . "<div style=\"display:none\">\n";
 }
@@ -1678,7 +1514,7 @@ sub printWarnings
 	{
 		ktWarn
 		(
-			"The following accessions look strange and may yield erroneous results. Please check if they are acual valid NCBI accessions:\n" .
+			"The following accessions were not in a valid format and were ignored:\n" .
 			join ' ', (keys %invalidAccs)
 		);
 		
@@ -1780,19 +1616,6 @@ sub setOption
 	$options{$option} = $value;
 }
 
-sub shouldCollapse
-{
-	my ($taxID) = @_;
-	
-	return !
-	(
-		getTaxRank($taxID) ne 'no rank' ||
-		! $options{'noRank'} && $taxID != 131567 ||
-		$taxID == 1 ||
-		$options{'cellular'} && $taxID == 131567
-	);
-}
-
 sub taxContains
 {
 	# determines if $parent is an ancestor of (or equal to) $child
@@ -1833,11 +1656,6 @@ sub taxLowestCommonAncestor
 	#
 	foreach my $node ( @nodes )
 	{
-		if ( $node == 1)
-		{
-		    return 1; # early out if any nodes are root
-		}
-		
 		while ( getTaxDepth($node) > $minDepth )
 		{
 			$node = getTaxParent($node);
@@ -2100,7 +1918,7 @@ sub dataHeader
 		my $enableText = $supp ? " enable=\"$suppDir/$suppEnableFile\"" : '';
 		$header .= indent(4) . "<$memberTag$enableText>members</$memberTag>\n";
 		$assignedText = " ${memberTag}Node=\"members\"";
-		$summaryText = " ${memberTag}All=\"members\"";
+		$summaryText = " ${memberTag}\All=\"members\"";
 		
 		if ( $options{'postUrl'} )
 		{
